import os
import unittest


if __name__ == '__main__':
    os.chdir(os.path.dirname(__file__))
    print(os.getcwd())

    for path in os.listdir('.'):
        if path.endswith('.py'):
            exec('from %s import *' % path[:-3])
    unittest.main()



#!/usr/bin/python3

import unittest
import os
import copy
import tempfile

import apt_pkg
import aptsources.sourceslist
import aptsources.distro

import testcommon


class TestAptSources(testcommon.TestCase):

    def setUp(self):
        testcommon.TestCase.setUp(self)
        if apt_pkg.config["APT::Architecture"] not in ('i386', 'amd64'):
            apt_pkg.config.set("APT::Architecture", "i386")
        apt_pkg.config.set("Dir::Etc", os.getcwd())
        apt_pkg.config.set("Dir::Etc::sourceparts", tempfile.mkdtemp())
        if os.path.exists("./build/data/templates"):
            self.templates = os.path.abspath("./build/data/templates")
        elif os.path.exists("../build/data/templates"):
            self.templates = os.path.abspath("../build/data/templates")
        else:
            self.templates = "/usr/share/python-apt/templates/"

    def tearDown(self):
        aptsources.distro._OSRelease.OS_RELEASE_FILE = \
            aptsources.distro._OSRelease.DEFAULT_OS_RELEASE_FILE
        if "LSB_ETC_LSB_RELEASE" in os.environ:
            del os.environ["LSB_ETC_LSB_RELEASE"]

    def testIsMirror(self):
        """aptsources: Test mirror detection."""
        yes = aptsources.sourceslist.is_mirror("http://archive.ubuntu.com",
                                               "http://de.archive.ubuntu.com")
        no = aptsources.sourceslist.is_mirror("http://archive.ubuntu.com",
                                              "http://ftp.debian.org")
        self.assertTrue(yes)
        self.assertFalse(no)

    def testSourcesListReading(self):
        """aptsources: Test sources.list parsing."""
        apt_pkg.config.set("Dir::Etc::sourcelist", "data/aptsources/"
                                                   "sources.list")
        sources = aptsources.sourceslist.SourcesList(True, self.templates)
        self.assertEqual(len(sources.list), 10)
        # test load
        sources.list = []
        sources.load("data/aptsources/sources.list")
        self.assertEqual(len(sources.list), 10)

    def testSourcesListAdding(self):
        """aptsources: Test additions to sources.list"""
        apt_pkg.config.set("Dir::Etc::sourcelist", "data/aptsources/"
                                                   "sources.list")
        sources = aptsources.sourceslist.SourcesList(True, self.templates)
        # test to add something that is already there (main)
        before = copy.deepcopy(sources)
        sources.add("deb", "http://de.archive.ubuntu.com/ubuntu/",
                    "edgy",
                    ["main"])
        self.assertTrue(sources.list == before.list)
        # test to add something that is already there (restricted)
        before = copy.deepcopy(sources)
        sources.add("deb", "http://de.archive.ubuntu.com/ubuntu/",
                    "edgy",
                    ["restricted"])
        self.assertTrue(sources.list == before.list)

        before = copy.deepcopy(sources)
        sources.add("deb", "http://de.archive.ubuntu.com/ubuntu/",
                    "natty",
                    ["main"], architectures=["amd64", "i386"])
        self.assertTrue(sources.list == before.list)

        # test to add something new: multiverse
        sources.add("deb", "http://de.archive.ubuntu.com/ubuntu/",
                    "edgy",
                    ["multiverse"])
        found = False
        for entry in sources:
            if (entry.type == "deb" and
                    entry.uri == "http://de.archive.ubuntu.com/ubuntu/" and
                    entry.dist == "edgy" and
                    "multiverse" in entry.comps):
                found = True
                break
        self.assertTrue(found)

        # add a new natty entry without architecture specification
        sources.add("deb", "http://de.archive.ubuntu.com/ubuntu/",
                    "natty",
                    ["multiverse"])
        found = False
        for entry in sources:
            if (entry.type == "deb" and
                    entry.uri == "http://de.archive.ubuntu.com/ubuntu/" and
                    entry.dist == "natty" and
                    entry.architectures == [] and
                    "multiverse" in entry.comps):
                found = True
                break
        self.assertTrue(found)

        # Add universe to existing multi-arch line
        sources.add("deb", "http://de.archive.ubuntu.com/ubuntu/",
                    "natty",
                    ["universe"], architectures=["i386", "amd64"])
        found = False
        for entry in sources:
            if (entry.type == "deb" and
                   entry.uri == "http://de.archive.ubuntu.com/ubuntu/" and
                   entry.dist == "natty" and
                   set(entry.architectures) == set(["amd64", "i386"]) and
                   set(entry.comps) == set(["main", "universe"])):
                found = True
                break
        self.assertTrue(found)
        # test to add something new: multiverse *and*
        # something that is already there
        before = copy.deepcopy(sources)
        sources.add("deb", "http://de.archive.ubuntu.com/ubuntu/",
                    "edgy",
                    ["universe", "something"])
        found_universe = 0
        found_something = 0
        for entry in sources:
            if (entry.type == "deb" and
                   entry.uri == "http://de.archive.ubuntu.com/ubuntu/" and
                   entry.dist == "edgy"):
                for c in entry.comps:
                    if c == "universe":
                        found_universe += 1
                    if c == "something":
                        found_something += 1
        #print "\n".join([s.str() for s in sources])
        self.assertEqual(found_something, 1)
        self.assertEqual(found_universe, 1)

    def testAddingWithComment(self):
        apt_pkg.config.set("Dir::Etc::sourcelist", "data/aptsources/"
                                                   "sources.list")
        sources = aptsources.sourceslist.SourcesList(True, self.templates)

        # test to add something that is already there (main); loses comment
        before = copy.deepcopy(sources)
        sources.add("deb", "http://de.archive.ubuntu.com/ubuntu/",
                    "edgy",
                    ["main"], comment="this will be lost")
        self.assertTrue(sources.list == before.list)
        for entry in sources:
            self.assertEqual(entry.comment, "")

        # test to add component to existing entry: multiverse; loses comment
        sources.add("deb", "http://de.archive.ubuntu.com/ubuntu/",
                    "edgy",
                    ["multiverse"], comment="this will be lost")
        for entry in sources:
            self.assertEqual(entry.comment, "")

        # test to add entirely new entry; retains comment
        sources.add("deb-src", "http://de.archive.ubuntu.com/ubuntu/",
                    "edgy",
                    ["main"], comment="this will appear")
        found = False
        for entry in sources:
            if (entry.type == "deb-src" and
                    entry.uri == "http://de.archive.ubuntu.com/ubuntu/" and
                    entry.dist == "edgy" and
                    entry.comment == "this will appear" and
                    "main" in entry.comps):
                found = True
                break
        self.assertTrue(found)

    def testInsertion(self):
        apt_pkg.config.set("Dir::Etc::sourcelist", "data/aptsources/"
                                                   "sources.list")
        sources = aptsources.sourceslist.SourcesList(True, self.templates)

        # test to insert something that is already there (universe); does not
        # move existing entry (remains at index 2)
        before = copy.deepcopy(sources)
        sources.add("deb", "http://de.archive.ubuntu.com/ubuntu/",
                    "edgy",
                    ["universe"], pos=0)
        self.assertTrue(sources.list == before.list)
        entry = list(sources)[5]
        self.assertTrue(
            entry.type == "deb" and
            entry.uri == "http://de.archive.ubuntu.com/ubuntu/" and
            entry.dist == "edgy" and
            "universe" in entry.comps)

        # test add component to existing entry: multiverse; does not move
        # entry to which it is appended (remains at index 0)
        sources.add("deb", "http://de.archive.ubuntu.com/ubuntu/",
                    "edgy",
                    ["multiverse"], pos=2)
        entry = list(sources)[1]
        self.assertTrue(
            entry.type == "deb" and
            entry.uri == "http://de.archive.ubuntu.com/ubuntu/" and
            entry.dist == "edgy" and
            {"main", "multiverse"} <= set(entry.comps))

        # test to add entirely new entry; inserted at 0
        sources.add("deb-src", "http://de.archive.ubuntu.com/ubuntu/",
                    "edgy",
                    ["main"], pos=0)
        entry = list(sources)[0]
        self.assertTrue(
            entry.type == "deb-src" and
            entry.uri == "http://de.archive.ubuntu.com/ubuntu/" and
            entry.dist == "edgy" and
            "main" in entry.comps)

    def testDuplication(self):
        apt_pkg.config.set("Dir::Etc::sourcelist",
                           "data/aptsources/sources.list.testDuplication")
        sources = aptsources.sourceslist.SourcesList(True, self.templates)
        test_url = "http://ppa.launchpad.net/me/myproject/ubuntu"
        # test to add something that is already there (enabled)
        before = copy.deepcopy(sources)
        sources.add("deb", test_url, "xenial", ["main"])
        self.assertTrue(sources.list == before.list)
        # test to add something that is already there (disabled)
        sources.add("# deb-src", test_url, "xenial", ["main"])
        self.assertTrue(sources.list == before.list)
        # test to enable something that is already there
        sources.add("deb-src", test_url, "xenial", ["main"])
        found = False
        self.assertEqual(len(sources.list), 2)
        for entry in sources:
            if (entry.type == "deb-src" and
                    not entry.disabled and
                    entry.uri == test_url and
                    entry.dist == "xenial" and
                    entry.architectures == [] and
                    entry.comps == ["main"]):
                found = True
                break
        self.assertTrue(found)

    def testMatcher(self):
        """aptsources: Test matcher"""
        apt_pkg.config.set("Dir::Etc::sourcelist", "data/aptsources/"
                           "sources.list.testDistribution")
        sources = aptsources.sourceslist.SourcesList(True, self.templates)
        distro = aptsources.distro.get_distro(id="Ubuntu",
                                              codename="bionic",
                                              description="Ubuntu 18.04 LTS",
                                              release="18.04")
        distro.get_sources(sources)
        # test if all suits of the current distro were detected correctly
        for s in sources:
            if not s.template:
                self.fail("source entry '%s' has no matcher" % s)

    def testMultiArch(self):
        """aptsources: Test multi-arch parsing"""

        apt_pkg.config.set("Dir::Etc::sourcelist", "data/aptsources/"
                           "sources.list")
        sources = aptsources.sourceslist.SourcesList(True, self.templates)
        assert not sources.list[8].invalid
        assert sources.list[8].type == "deb"
        assert sources.list[8].architectures == ["amd64", "i386"]
        assert sources.list[8].uri == "http://de.archive.ubuntu.com/ubuntu/"
        assert sources.list[8].dist == "natty"
        assert sources.list[8].comps == ["main"]
        assert sources.list[8].line.strip() == str(sources.list[8])
        assert sources.list[8].trusted is None

    def testMultipleOptions(self):
        """aptsources: Test multi-arch parsing"""

        apt_pkg.config.set("Dir::Etc::sourcelist", "data/aptsources/"
                           "sources.list")
        sources = aptsources.sourceslist.SourcesList(True, self.templates)
        assert sources.list[9].invalid is False
        assert sources.list[9].type == "deb"
        assert sources.list[9].architectures == ["amd64", "i386"]
        self.assertEqual(
            sources.list[9].uri, "http://de.archive.ubuntu.com/ubuntu/")
        assert sources.list[9].dist == "natty"
        assert sources.list[9].comps == ["main"]
        assert sources.list[9].trusted
        assert sources.list[9].line.strip() == str(sources.list[9])

    def test_enable_component(self):
        target = "./data/aptsources/sources.list.enable_comps"
        line = "deb http://archive.ubuntu.com/ubuntu lucid main\n"
        with open(target, "w") as target_file:
            target_file.write(line)
        apt_pkg.config.set("Dir::Etc::sourcelist", target)
        sources = aptsources.sourceslist.SourcesList(True, self.templates)
        distro = aptsources.distro.get_distro(id="Ubuntu")
        # make sure we are using the right distro
        distro.codename = "lucid"
        distro.id = "Ubuntu"
        distro.release = "10.04"
        # and get the sources
        distro.get_sources(sources)
        # test enable_component
        comp = "multiverse"
        distro.enable_component(comp)
        comps = set()
        for entry in sources:
            comps = comps.union(set(entry.comps))
        self.assertTrue("multiverse" in comps)
        self.assertTrue("universe" in comps)

    def testDistribution(self):
        """aptsources: Test distribution detection."""
        apt_pkg.config.set("Dir::Etc::sourcelist", "data/aptsources/"
                           "sources.list.testDistribution")
        sources = aptsources.sourceslist.SourcesList(True, self.templates)
        distro = aptsources.distro.get_distro(id="Ubuntu",
                                              codename="bionic",
                                              description="Ubuntu 18.04 LTS",
                                              release="18.04")
        distro.get_sources(sources)
        # test if all suits of the current distro were detected correctly
        dist_templates = set()
        for s in sources:
            if s.template:
                dist_templates.add(s.template.name)
        #print dist_templates
        for d in ("hardy", "hardy-security", "hardy-updates", "intrepid",
                  "hardy-backports"):
            self.assertTrue(d in dist_templates)
        # test enable
        comp = "restricted"
        distro.enable_component(comp)
        found = {}
        for entry in sources:
            if (entry.type == "deb" and
                    entry.uri == "http://de.archive.ubuntu.com/ubuntu/" and
                    "edgy" in entry.dist):
                for c in entry.comps:
                    if c == comp:
                        if entry.dist not in found:
                            found[entry.dist] = 0
                        found[entry.dist] += 1
        #print "".join([s.str() for s in sources])
        for key in found:
            self.assertEqual(found[key], 1)

        # add a not-already available component
        comp = "multiverse"
        distro.enable_component(comp)
        found = {}
        for entry in sources:
            if (entry.type == "deb" and
                    entry.template and
                    entry.template.name == "edgy"):
                for c in entry.comps:
                    if c == comp:
                        if entry.dist not in found.has_key:
                            found[entry.dist] = 0
                        found[entry.dist] += 1
        #print "".join([s.str() for s in sources])
        for key in found:
            self.assertEqual(found[key], 1)

    @unittest.skip("lsb-release test broken when it was added")
    def test_os_release_distribution(self):
        """os-release file can be read and is_like is populated accordingly"""
        os.environ["LSB_ETC_LSB_RELEASE"] = \
            os.path.abspath("./data/aptsources/lsb-release")
        aptsources.distro._OSRelease.OS_RELEASE_FILE = \
            os.path.abspath("./data/aptsources/os-release")
        distro = aptsources.distro.get_distro()
        # Everything but is_like comes from lsb_release, see TODO in
        # get_distro.
        self.assertEqual('Ubuntu', distro.id)
        self.assertEqual('xenial', distro.codename)
        self.assertEqual('Ubuntu 16.04.1 LTS', distro.description)
        self.assertEqual('16.04', distro.release)
        self.assertEqual(['ubuntu', 'debian'], distro.is_like)

    def test_enable_disabled(self):
        """LP: #1042916: Test enabling disabled entry."""
        apt_pkg.config.set("Dir::Etc::sourcelist", "data/aptsources/"
                           "sources.list")
        sources = aptsources.sourceslist.SourcesList(True, self.templates)
        disabled = sources.add("deb", "http://fi.archive.ubuntu.com/ubuntu/",
                    "precise",
                    ["main"])
        disabled.set_enabled(False)
        enabled = sources.add("deb", "http://fi.archive.ubuntu.com/ubuntu/",
                    "precise",
                    ["main"])
        self.assertEqual(disabled, enabled)
        self.assertFalse(disabled.disabled)

    def test_duplicate_uri_with_trailing_slash(self):
        """Test replacing entry with same uri except trailing slash"""
        apt_pkg.config.set("Dir::Etc::sourcelist", "data/aptsources/"
                           "sources.list")
        sources = aptsources.sourceslist.SourcesList(True, self.templates)
        line_wslash = "deb http://rslash.ubuntu.com/ubuntu/ precise main"
        line_woslash = "deb http://rslash.ubuntu.com/ubuntu precise main"
        entry_wslash = aptsources.sourceslist.SourceEntry(line_wslash)
        entry_woslash = aptsources.sourceslist.SourceEntry(line_woslash)
        self.assertEqual(entry_wslash, entry_woslash)
        count = len(sources.list)
        sourceslist_wslash = sources.add(entry_wslash.type,
                                         entry_wslash.uri,
                                         entry_wslash.dist,
                                         entry_wslash.comps)
        self.assertEqual(count + 1, len(sources.list))
        count = len(sources.list)
        sourceslist_woslash = sources.add(entry_woslash.type,
                                          entry_woslash.uri,
                                          entry_woslash.dist,
                                          entry_woslash.comps)
        self.assertEqual(count, len(sources.list))
        self.assertEqual(sourceslist_wslash, sourceslist_woslash)


if __name__ == "__main__":
    os.chdir(os.path.dirname(__file__))
    unittest.main()



#!/usr/bin/python3

import apt_pkg
apt_pkg.init()

cache = apt_pkg.Cache()


class Wanted:

    def __init__(self, name):
        self.name = name
        self.recommended = []
        self.suggested = []


wanted = {}

for package in cache.packages:
    current = package.current_ver
    if not current:
        continue
    depends = current.depends_list
    for (key, attr) in (('Suggests', 'suggested'),
                        ('Recommends', 'recommended')):
        list = depends.get(key, [])
        for dependency in list:
            name = dependency[0].target_pkg.name
            dep = cache[name]
            if dep.current_ver:
                continue
            getattr(wanted.setdefault(name, Wanted(name)),
                    attr).append(package.name)

ks = list(wanted.keys())
ks.sort()

for want in ks:
    print(want, wanted[want].recommended, wanted[want].suggested)



#!/usr/bin/python3

import apt_pkg

apt_pkg.init()

#cache = apt_pkg.Cache()
#sources = apt_pkg.SourceRecords(cache)

sources = apt_pkg.SourceRecords()
sources.restart()
while sources.lookup('hello'):
    print(sources.package, sources.version, sources.maintainer,
        sources.section, repr(sources.binaries))
    print(sources.files)
    print(sources.index.archive_uri(sources.files[0][2]))



# cache.py - apt cache abstraction
#
#  Copyright (c) 2005-2009 Canonical
#
#  Author: Michael Vogt <michael.vogt@ubuntu.com>
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation; either version 2 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA

from __future__ import print_function

import fnmatch
import os
import warnings
import weakref

try:
    from typing import (Any, Callable, Dict, Iterator, List, Optional,
                        Set, Tuple, Union, cast, KeysView)
    Any  # pyflakes
    Callable  # pyflakes
    Dict  # pyflakes
    Iterator  # pyflakes
    KeysView  # pyflakes
    List  # pyflakes
    Optional  # pyflakes
    Set  # pyflakes
    Tuple  # pyflakes
    Union  # pyflakes
except ImportError:
    def cast(typ, obj):  # type: ignore
        return obj
    pass

import apt_pkg
from apt.package import Package, Version
import apt.progress.text
from apt.progress.base import AcquireProgress, InstallProgress, OpProgress
OpProgress  # pyflakes
InstallProgress  # pyflakes
AcquireProgress  # pyflakes
Version  # pyflakes


class FetchCancelledException(IOError):
    """Exception that is thrown when the user cancels a fetch operation."""


class FetchFailedException(IOError):
    """Exception that is thrown when fetching fails."""


class UntrustedException(FetchFailedException):
    """Exception that is thrown when fetching fails for trust reasons"""


class LockFailedException(IOError):
    """Exception that is thrown when locking fails."""


class CacheClosedException(Exception):
    """Exception that is thrown when the cache is used after close()."""


class _WrappedLock(object):
    """Wraps an apt_pkg.FileLock to raise LockFailedException.

    Initialized using a directory path."""

    def __init__(self, path):
        # type: (str) -> None
        self._path = path
        self._lock = apt_pkg.FileLock(os.path.join(path, "lock"))

    def __enter__(self):
        # type: () -> None
        try:
            return self._lock.__enter__()
        except apt_pkg.Error as e:
            raise LockFailedException(("Failed to lock directory %s: %s") %
                                       (self._path, e))

    def __exit__(self, typ, value, traceback):
        # type: (object, object, object) -> None
        return self._lock.__exit__(typ, value, traceback)


class Cache(object):
    """Dictionary-like package cache.

    The APT cache file contains a hash table mapping names of binary
    packages to their metadata. A Cache object is the in-core
    representation of the same. It provides access to APTs idea of the
    list of available packages.

    The cache can be used like a mapping from package names to Package
    objects (although only getting items is supported).

    Keyword arguments:
    progress -- a OpProgress object,
    rootdir  -- an alternative root directory. if that is given the system
    sources.list and system lists/files are not read, only file relative
    to the given rootdir,
    memonly  -- build the cache in memory only.


    .. versionchanged:: 1.0

        The cache now supports package names with special architecture
        qualifiers such as :all and :native. It does not export them
        in :meth:`keys()`, though, to keep :meth:`keys()` a unique set.
    """

    def __init__(self, progress=None, rootdir=None, memonly=False):
        # type: (Optional[OpProgress], Optional[str], bool) -> None
        self._cache = cast(apt_pkg.Cache, None)  # type: apt_pkg.Cache
        self._depcache = cast(apt_pkg.DepCache, None)  # type: apt_pkg.DepCache
        self._records = cast(apt_pkg.PackageRecords, None)  # type: apt_pkg.PackageRecords # noqa
        self._list = cast(apt_pkg.SourceList, None)  # type: apt_pkg.SourceList
        self._callbacks = {}  # type: Dict[str, List[Union[Callable[..., None],str]]] # noqa
        self._callbacks2 = {}  # type: Dict[str, List[Tuple[Callable[..., Any], Tuple[Any, ...], Dict[Any,Any]]]] # noqa
        self._weakref = weakref.WeakValueDictionary()  # type: weakref.WeakValueDictionary[str, apt.Package] # noqa
        self._weakversions = weakref.WeakSet()  # type: weakref.WeakSet[Version] # noqa
        self._changes_count = -1
        self._sorted_set = None  # type: Optional[List[str]]

        self.connect("cache_post_open", "_inc_changes_count")
        self.connect("cache_post_change", "_inc_changes_count")
        if memonly:
            # force apt to build its caches in memory
            apt_pkg.config.set("Dir::Cache::pkgcache", "")
        if rootdir:
            rootdir = os.path.abspath(rootdir)
            if os.path.exists(rootdir + "/etc/apt/apt.conf"):
                apt_pkg.read_config_file(apt_pkg.config,
                                         rootdir + "/etc/apt/apt.conf")
            if os.path.isdir(rootdir + "/etc/apt/apt.conf.d"):
                apt_pkg.read_config_dir(apt_pkg.config,
                                        rootdir + "/etc/apt/apt.conf.d")
            apt_pkg.config.set("Dir", rootdir)
            apt_pkg.config.set("Dir::State::status",
                               rootdir + "/var/lib/dpkg/status")
            # also set dpkg to the rootdir path so that its called for the
            # --print-foreign-architectures call
            apt_pkg.config.set("Dir::bin::dpkg",
                               os.path.join(rootdir, "usr", "bin", "dpkg"))
            # create required dirs/files when run with special rootdir
            # automatically
            self._check_and_create_required_dirs(rootdir)
            # Call InitSystem so the change to Dir::State::Status is actually
            # recognized (LP: #320665)
            apt_pkg.init_system()

        # Prepare a lock object (context manager for archive lock)
        archive_dir = apt_pkg.config.find_dir("Dir::Cache::Archives")
        self._archive_lock = _WrappedLock(archive_dir)

        self.open(progress)

    def fix_broken(self):
        # type: () -> None
        """Fix broken packages."""
        self._depcache.fix_broken()

    def _inc_changes_count(self):
        # type: () -> None
        """Increase the number of changes"""
        self._changes_count += 1

    def _check_and_create_required_dirs(self, rootdir):
        # type: (str) -> None
        """
        check if the required apt directories/files are there and if
        not create them
        """
        files = [
            "/var/lib/dpkg/status",
            "/etc/apt/sources.list",
        ]
        dirs = [
            "/var/lib/dpkg",
            "/etc/apt/",
            "/var/cache/apt/archives/partial",
            "/var/lib/apt/lists/partial",
        ]
        for d in dirs:
            if not os.path.exists(rootdir + d):
                #print "creating: ", rootdir + d
                os.makedirs(rootdir + d)
        for f in files:
            if not os.path.exists(rootdir + f):
                open(rootdir + f, "w").close()

    def _run_callbacks(self, name):
        # type: (str) -> None
        """ internal helper to run a callback """
        if name in self._callbacks:
            for callback in self._callbacks[name]:
                if callback == '_inc_changes_count':
                    self._inc_changes_count()
                else:
                    callback()  # type: ignore

        if name in self._callbacks2:
            for callback, args, kwds in self._callbacks2[name]:
                callback(self, *args, **kwds)

    def open(self, progress=None):
        # type: (Optional[OpProgress]) -> None
        """ Open the package cache, after that it can be used like
            a dictionary
        """
        if progress is None:
            progress = apt.progress.base.OpProgress()
        # close old cache on (re)open
        self.close()
        self.op_progress = progress
        self._run_callbacks("cache_pre_open")

        self._cache = apt_pkg.Cache(progress)
        self._depcache = apt_pkg.DepCache(self._cache)
        self._records = apt_pkg.PackageRecords(self._cache)
        self._list = apt_pkg.SourceList()
        self._list.read_main_list()
        self._sorted_set = None
        self.__remap()

        self._have_multi_arch = len(apt_pkg.get_architectures()) > 1

        progress.done()
        self._run_callbacks("cache_post_open")

    def __remap(self):
        # type: () -> None
        """Called after cache reopen() to relocate to new cache.

        Relocate objects like packages and versions from the old
        underlying cache to the new one.
        """
        for key in list(self._weakref.keys()):
            try:
                pkg = self._weakref[key]
            except KeyError:
                continue

            try:
                pkg._pkg = self._cache[pkg._pkg.name, pkg._pkg.architecture]
            except LookupError:
                del self._weakref[key]

        for ver in list(self._weakversions):
            # Package has been reseated above, reseat version
            for v in ver.package._pkg.version_list:
                # Requirements as in debListParser::SameVersion
                if (v.hash == ver._cand.hash and
                    (v.size == 0 or ver._cand.size == 0 or
                     v.size == ver._cand.size) and
                    v.multi_arch == ver._cand.multi_arch and
                    v.ver_str == ver._cand.ver_str):
                    ver._cand = v
                    break
            else:
                self._weakversions.remove(ver)

    def close(self):
        # type: () -> None
        """ Close the package cache """
        # explicitely free the FDs that _records has open
        del self._records
        self._records = cast(apt_pkg.PackageRecords, None)

    def __enter__(self):
        # type: () -> Cache
        """ Enter the with statement """
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        # type: (object, object, object) -> None
        """ Exit the with statement """
        self.close()

    def __getitem__(self, key):
        # type: (object) -> Package
        """ look like a dictionary (get key) """
        try:
            key = str(key)
            rawpkg = self._cache[key]
        except KeyError:
            raise KeyError('The cache has no package named %r' % key)

        # It might be excluded due to not having a version or something
        if not self.__is_real_pkg(rawpkg):
            raise KeyError('The cache has no package named %r' % key)

        pkg = self._rawpkg_to_pkg(rawpkg)

        return pkg

    def get(self, key, default=None):
        # type: (object, object) -> Any
        """Return *self*[*key*] or *default* if *key* not in *self*.

        .. versionadded:: 1.1
        """
        try:
            return self[key]
        except KeyError:
            return default

    def _rawpkg_to_pkg(self, rawpkg):
        # type: (apt_pkg.Package) -> Package
        """Returns the apt.Package object for an apt_pkg.Package object.

        .. versionadded:: 1.0.0
        """
        fullname = rawpkg.get_fullname(pretty=True)

        return self._weakref.setdefault(fullname, Package(self, rawpkg))

    def __iter__(self):
        # type: () -> Iterator[Package]
        # We iterate sorted over package names here. With this we read the
        # package lists linearly if we need to access the package records,
        # instead of having to do thousands of random seeks; the latter
        # is disastrous if we use compressed package indexes, and slower than
        # necessary for uncompressed indexes.
        for pkgname in self.keys():
            pkg = Package(self, self._cache[pkgname])
            yield self._weakref.setdefault(pkgname, pkg)

    def __is_real_pkg(self, rawpkg):
        # type: (apt_pkg.Package) -> bool
        """Check if the apt_pkg.Package provided is a real package."""
        return rawpkg.has_versions

    def has_key(self, key):
        # type: (object) -> bool
        return key in self

    def __contains__(self, key):
        # type: (object) -> bool
        try:
            return self.__is_real_pkg(self._cache[str(key)])
        except KeyError:
            return False

    def __len__(self):
        # type: () -> int
        return len(self.keys())

    def keys(self):
        # type: () -> List[str]
        if self._sorted_set is None:
            self._sorted_set = sorted(p.get_fullname(pretty=True)
                                      for p in self._cache.packages
                                        if self.__is_real_pkg(p))
        return list(self._sorted_set)  # We need a copy here, caller may modify

    def get_changes(self):
        # type: () -> List[Package]
        """ Get the marked changes """
        changes = []
        marked_keep = self._depcache.marked_keep
        for rawpkg in self._cache.packages:
            if not marked_keep(rawpkg):
                changes.append(self._rawpkg_to_pkg(rawpkg))
        return changes

    def upgrade(self, dist_upgrade=False):
        # type: (bool) -> None
        """Upgrade all packages.

        If the parameter *dist_upgrade* is True, new dependencies will be
        installed as well (and conflicting packages may be removed). The
        default value is False.
        """
        self.cache_pre_change()
        self._depcache.upgrade(dist_upgrade)
        self.cache_post_change()

    @property
    def required_download(self):
        # type: () -> int
        """Get the size of the packages that are required to download."""
        if self._records is None:
            raise CacheClosedException(
                "Cache object used after close() called")
        pm = apt_pkg.PackageManager(self._depcache)
        fetcher = apt_pkg.Acquire()
        pm.get_archives(fetcher, self._list, self._records)
        return fetcher.fetch_needed

    @property
    def required_space(self):
        # type: () -> int
        """Get the size of the additional required space on the fs."""
        return self._depcache.usr_size

    @property
    def req_reinstall_pkgs(self):
        # type: () -> Set[str]
        """Return the packages not downloadable packages in reqreinst state."""
        reqreinst = set()
        get_candidate_ver = self._depcache.get_candidate_ver
        states = frozenset((apt_pkg.INSTSTATE_REINSTREQ,
                            apt_pkg.INSTSTATE_HOLD_REINSTREQ))
        for pkg in self._cache.packages:
            cand = get_candidate_ver(pkg)
            if cand and not cand.downloadable and pkg.inst_state in states:
                reqreinst.add(pkg.get_fullname(pretty=True))
        return reqreinst

    def _run_fetcher(self, fetcher, allow_unauthenticated):
        # type: (apt_pkg.Acquire, Optional[bool]) -> int
        if allow_unauthenticated is None:
            allow_unauthenticated = apt_pkg.config.find_b("APT::Get::"
                                        "AllowUnauthenticated", False)

        untrusted = [item for item in fetcher.items if not item.is_trusted]
        if untrusted and not allow_unauthenticated:
            raise UntrustedException("Untrusted packages:\n%s" %
                "\n".join(i.desc_uri for i in untrusted))

        # do the actual fetching
        res = fetcher.run()

        # now check the result (this is the code from apt-get.cc)
        failed = False
        err_msg = ""
        for item in fetcher.items:
            if item.status == item.STAT_DONE:
                continue
            if item.STAT_IDLE:
                continue
            err_msg += "Failed to fetch %s %s\n" % (item.desc_uri,
                                                    item.error_text)
            failed = True

        # we raise a exception if the download failed or it was cancelt
        if res == fetcher.RESULT_CANCELLED:
            raise FetchCancelledException(err_msg)
        elif failed:
            raise FetchFailedException(err_msg)
        return res

    def _fetch_archives(self,
                        fetcher,                # type: apt_pkg.Acquire
                        pm,                     # type: apt_pkg.PackageManager
                        allow_unauthenticated=None,  # type: Optional[bool]
                        ):
        # type: (...) -> int
        """ fetch the needed archives """
        if self._records is None:
            raise CacheClosedException(
                "Cache object used after close() called")

        # this may as well throw a SystemError exception
        if not pm.get_archives(fetcher, self._list, self._records):
            return False

        # now run the fetcher, throw exception if something fails to be
        # fetched
        return self._run_fetcher(fetcher, allow_unauthenticated)

    def fetch_archives(self,
                       progress=None,        # type: Optional[AcquireProgress]
                       fetcher=None,         # type: Optional[apt_pkg.Acquire]
                       allow_unauthenticated=None,  # type: Optional[bool]
                       ):
        # type: (...) -> int
        """Fetch the archives for all packages marked for install/upgrade.

        You can specify either an :class:`apt.progress.base.AcquireProgress()`
        object for the parameter *progress*, or specify an already
        existing :class:`apt_pkg.Acquire` object for the parameter *fetcher*.

        The return value of the function is undefined. If an error occurred,
        an exception of type :class:`FetchFailedException` or
        :class:`FetchCancelledException` is raised.

        The keyword-only parameter *allow_unauthenticated* specifies whether
        to allow unauthenticated downloads. If not specified, it defaults to
        the configuration option `APT::Get::AllowUnauthenticated`.

        .. versionadded:: 0.8.0
        """
        if progress is not None and fetcher is not None:
            raise ValueError("Takes a progress or a an Acquire object")
        if progress is None:
            progress = apt.progress.text.AcquireProgress()
        if fetcher is None:
            fetcher = apt_pkg.Acquire(progress)

        with self._archive_lock:
            return self._fetch_archives(fetcher,
                                        apt_pkg.PackageManager(self._depcache),
                                        allow_unauthenticated)

    def is_virtual_package(self, pkgname):
        # type: (str) -> bool
        """Return whether the package is a virtual package."""
        try:
            pkg = self._cache[pkgname]
        except KeyError:
            return False
        else:
            return bool(pkg.has_provides and not pkg.has_versions)

    def get_providing_packages(self, pkgname, candidate_only=True,
                               include_nonvirtual=False):
        # type: (str, bool, bool) -> List[Package]
        """Return a list of all packages providing a package.

        Return a list of packages which provide the virtual package of the
        specified name.

        If 'candidate_only' is False, return all packages with at
        least one version providing the virtual package. Otherwise,
        return only those packages where the candidate version
        provides the virtual package.

        If 'include_nonvirtual' is True then it will search for all
        packages providing pkgname, even if pkgname is not itself
        a virtual pkg.
        """

        providers = set()  # type: Set[Package]
        get_candidate_ver = self._depcache.get_candidate_ver
        try:
            vp = self._cache[pkgname]
            if vp.has_versions and not include_nonvirtual:
                return list(providers)
        except KeyError:
            return list(providers)

        for provides, providesver, version in vp.provides_list:
            rawpkg = version.parent_pkg
            if not candidate_only or (version == get_candidate_ver(rawpkg)):
                providers.add(self._rawpkg_to_pkg(rawpkg))
        return list(providers)

    def update(self, fetch_progress=None, pulse_interval=0,
               raise_on_error=True, sources_list=None):
        # type: (Optional[AcquireProgress], int, bool, Optional[str]) -> int
        """Run the equivalent of apt-get update.

        You probably want to call open() afterwards, in order to utilise the
        new cache. Otherwise, the old cache will be used which can lead to
        strange bugs.

        The first parameter *fetch_progress* may be set to an instance of
        apt.progress.FetchProgress, the default is apt.progress.FetchProgress()
        .
        sources_list -- Update a alternative sources.list than the default.
        Note that the sources.list.d directory is ignored in this case
        """
        with _WrappedLock(apt_pkg.config.find_dir("Dir::State::Lists")):
            if sources_list:
                old_sources_list = apt_pkg.config.find("Dir::Etc::sourcelist")
                old_sources_list_d = (
                    apt_pkg.config.find("Dir::Etc::sourceparts"))
                old_cleanup = apt_pkg.config.find("APT::List-Cleanup")
                apt_pkg.config.set("Dir::Etc::sourcelist",
                                   os.path.abspath(sources_list))
                apt_pkg.config.set("Dir::Etc::sourceparts", "xxx")
                apt_pkg.config.set("APT::List-Cleanup", "0")
                slist = apt_pkg.SourceList()
                slist.read_main_list()
            else:
                slist = self._list

            try:
                if fetch_progress is None:
                    fetch_progress = apt.progress.base.AcquireProgress()
                try:
                    res = self._cache.update(fetch_progress, slist,
                                             pulse_interval)
                except SystemError as e:
                    raise FetchFailedException(e)
                if not res and raise_on_error:
                    raise FetchFailedException()
                else:
                    return res
            finally:
                if sources_list:
                    apt_pkg.config.set("Dir::Etc::sourcelist",
                                       old_sources_list)
                    apt_pkg.config.set("Dir::Etc::sourceparts",
                                       old_sources_list_d)
                    apt_pkg.config.set("APT::List-Cleanup",
                                       old_cleanup)

    def install_archives(self, pm, install_progress):
        # type: (apt_pkg.PackageManager, InstallProgress) -> int
        """
        The first parameter *pm* refers to an object returned by
        apt_pkg.PackageManager().

        The second parameter *install_progress* refers to an InstallProgress()
        object of the module apt.progress.

        This releases a system lock in newer versions, if there is any,
        and reestablishes it afterwards.
        """
        # compat with older API
        try:
            install_progress.startUpdate()  # type: ignore
        except AttributeError:
            install_progress.start_update()

        did_unlock = apt_pkg.pkgsystem_is_locked()
        if did_unlock:
            apt_pkg.pkgsystem_unlock_inner()

        try:
            res = install_progress.run(pm)
        finally:
            if did_unlock:
                apt_pkg.pkgsystem_lock_inner()

        try:
            install_progress.finishUpdate()  # type: ignore
        except AttributeError:
            install_progress.finish_update()
        return res

    def commit(self,
               fetch_progress=None,         # type: Optional[AcquireProgress]
               install_progress=None,       # type: Optional[InstallProgress]
               allow_unauthenticated=None,  # type: Optional[bool]
               ):
        # type: (...) -> bool
        """Apply the marked changes to the cache.

        The first parameter, *fetch_progress*, refers to a FetchProgress()
        object as found in apt.progress, the default being
        apt.progress.FetchProgress().

        The second parameter, *install_progress*, is a
        apt.progress.InstallProgress() object.

        The keyword-only parameter *allow_unauthenticated* specifies whether
        to allow unauthenticated downloads. If not specified, it defaults to
        the configuration option `APT::Get::AllowUnauthenticated`.
        """
        # FIXME:
        # use the new acquire/pkgmanager interface here,
        # raise exceptions when a download or install fails
        # and send proper error strings to the application.
        # Current a failed download will just display "error"
        # which is less than optimal!

        if fetch_progress is None:
            fetch_progress = apt.progress.base.AcquireProgress()
        if install_progress is None:
            install_progress = apt.progress.base.InstallProgress()

        assert install_progress is not None

        with apt_pkg.SystemLock():
            pm = apt_pkg.PackageManager(self._depcache)
            fetcher = apt_pkg.Acquire(fetch_progress)
            with self._archive_lock:
                while True:
                    # fetch archives first
                    res = self._fetch_archives(fetcher, pm,
                                               allow_unauthenticated)

                    # then install
                    res = self.install_archives(pm, install_progress)
                    if res == pm.RESULT_COMPLETED:
                        break
                    elif res == pm.RESULT_FAILED:
                        raise SystemError("installArchives() failed")
                    elif res == pm.RESULT_INCOMPLETE:
                        pass
                    else:
                        raise SystemError("internal-error: unknown result "
                                          "code from InstallArchives: %s" %
                                          res)
                    # reload the fetcher for media swaping
                    fetcher.shutdown()
        return (res == pm.RESULT_COMPLETED)

    def clear(self):
        # type: () -> None
        """ Unmark all changes """
        self._depcache.init()

    # cache changes

    def cache_post_change(self):
        # type: () -> None
        " called internally if the cache has changed, emit a signal then "
        self._run_callbacks("cache_post_change")

    def cache_pre_change(self):
        # type: () -> None
        """ called internally if the cache is about to change, emit
            a signal then """
        self._run_callbacks("cache_pre_change")

    def connect(self, name, callback):
        # type: (str, Union[Callable[..., None],str]) -> None
        """Connect to a signal.

        .. deprecated:: 1.0

            Please use connect2() instead, as this function is very
            likely to cause a memory leak.
        """
        if callback != '_inc_changes_count':
            warnings.warn("connect() likely causes a reference"
                          " cycle, use connect2() instead", RuntimeWarning, 2)
        if name not in self._callbacks:
            self._callbacks[name] = []
        self._callbacks[name].append(callback)

    def connect2(self, name, callback, *args, **kwds):
        # type: (str, Callable[..., Any], object, object) -> None
        """Connect to a signal.

        The callback will be passed the cache as an argument, and
        any arguments passed to this function. Make sure that, if you
        pass a method of a class as your callback, your class does not
        contain a reference to the cache.

        Cyclic references to the cache can cause issues if the Cache object
        is replaced by a new one, because the cache keeps a lot of objects and
        tens of open file descriptors.

        currently only used for cache_{post,pre}_{changed,open}.

        .. versionadded:: 1.0
        """
        if name not in self._callbacks2:
            self._callbacks2[name] = []
        self._callbacks2[name].append((callback, args, kwds))

    def actiongroup(self):
        # type: () -> apt_pkg.ActionGroup
        """Return an `ActionGroup` object for the current cache.

        Action groups can be used to speedup actions. The action group is
        active as soon as it is created, and disabled when the object is
        deleted or when release() is called.

        You can use the action group as a context manager, this is the
        recommended way::

            with cache.actiongroup():
                for package in my_selected_packages:
                    package.mark_install()

        This way, the action group is automatically released as soon as the
        with statement block is left. It also has the benefit of making it
        clear which parts of the code run with a action group and which
        don't.
        """
        return apt_pkg.ActionGroup(self._depcache)

    @property
    def dpkg_journal_dirty(self):
        # type: () -> bool
        """Return True if the dpkg was interrupted

        All dpkg operations will fail until this is fixed, the action to
        fix the system if dpkg got interrupted is to run
        'dpkg --configure -a' as root.
        """
        dpkg_status_dir = os.path.dirname(
            apt_pkg.config.find_file("Dir::State::status"))
        for f in os.listdir(os.path.join(dpkg_status_dir, "updates")):
            if fnmatch.fnmatch(f, "[0-9]*"):
                return True
        return False

    @property
    def broken_count(self):
        # type: () -> int
        """Return the number of packages with broken dependencies."""
        return self._depcache.broken_count

    @property
    def delete_count(self):
        # type: () -> int
        """Return the number of packages marked for deletion."""
        return self._depcache.del_count

    @property
    def install_count(self):
        # type: () -> int
        """Return the number of packages marked for installation."""
        return self._depcache.inst_count

    @property
    def keep_count(self):
        # type: () -> int
        """Return the number of packages marked as keep."""
        return self._depcache.keep_count


class ProblemResolver(object):
    """Resolve problems due to dependencies and conflicts.

    The first argument 'cache' is an instance of apt.Cache.
    """

    def __init__(self, cache):
        # type: (Cache) -> None
        self._resolver = apt_pkg.ProblemResolver(cache._depcache)
        self._cache = cache

    def clear(self, package):
        # type: (Package) -> None
        """Reset the package to the default state."""
        self._resolver.clear(package._pkg)

    def protect(self, package):
        # type: (Package) -> None
        """Protect a package so it won't be removed."""
        self._resolver.protect(package._pkg)

    def remove(self, package):
        # type: (Package) -> None
        """Mark a package for removal."""
        self._resolver.remove(package._pkg)

    def resolve(self):
        # type: () -> None
        """Resolve dependencies, try to remove packages where needed."""
        self._cache.cache_pre_change()
        self._resolver.resolve()
        self._cache.cache_post_change()

    def resolve_by_keep(self):
        # type: () -> None
        """Resolve dependencies, do not try to remove packages."""
        self._cache.cache_pre_change()
        self._resolver.resolve_by_keep()
        self._cache.cache_post_change()


# ----------------------------- experimental interface


class Filter(object):
    """ Filter base class """

    def apply(self, pkg):
        # type: (Package) -> bool
        """ Filter function, return True if the package matchs a
            filter criteria and False otherwise
        """
        return True


class MarkedChangesFilter(Filter):
    """ Filter that returns all marked changes """

    def apply(self, pkg):
        # type: (Package) -> bool
        if pkg.marked_install or pkg.marked_delete or pkg.marked_upgrade:
            return True
        else:
            return False


class InstalledFilter(Filter):
    """Filter that returns all installed packages.

    .. versionadded:: 1.0.0
    """

    def apply(self, pkg):
        # type: (Package) -> bool
        return pkg.is_installed


class _FilteredCacheHelper(object):
    """Helper class for FilteredCache to break a reference cycle."""

    def __init__(self, cache):
        # type: (Cache) -> None
        # Do not keep a reference to the cache, or you have a cycle!

        self._filtered = {}  # type: Dict[str,bool]
        self._filters = []  # type: List[Filter]
        cache.connect2("cache_post_change", self.filter_cache_post_change)
        cache.connect2("cache_post_open", self.filter_cache_post_change)

    def _reapply_filter(self, cache):
        # type: (Cache) -> None
        " internal helper to refilter "
        # Do not keep a reference to the cache, or you have a cycle!
        self._filtered = {}
        for pkg in cache:
            for f in self._filters:
                if f.apply(pkg):
                    self._filtered[pkg.name] = True
                    break

    def set_filter(self, filter):
        # type: (Filter) -> None
        """Set the current active filter."""
        self._filters = []
        self._filters.append(filter)

    def filter_cache_post_change(self, cache):
        # type: (Cache) -> None
        """Called internally if the cache changes, emit a signal then."""
        # Do not keep a reference to the cache, or you have a cycle!
        self._reapply_filter(cache)


class FilteredCache(object):
    """ A package cache that is filtered.

        Can work on a existing cache or create a new one
    """

    def __init__(self, cache=None, progress=None):
        # type: (Optional[Cache], Optional[OpProgress]) -> None
        if cache is None:
            self.cache = Cache(progress)
        else:
            self.cache = cache
        self._helper = _FilteredCacheHelper(self.cache)

    def __len__(self):
        # type: () -> int
        return len(self._helper._filtered)

    def __getitem__(self, key):
        # type: (str) -> Package
        return self.cache[key]

    def __iter__(self):
        # type: () -> Iterator[Package]
        for pkgname in self._helper._filtered:
            yield self.cache[pkgname]

    def keys(self):
        # type: () -> KeysView[str]
        return self._helper._filtered.keys()

    def has_key(self, key):
        # type: (object) -> bool
        return key in self

    def __contains__(self, key):
        # type: (object) -> bool
        try:
            # Normalize package name for multi arch
            return self.cache[key].name in self._helper._filtered
        except KeyError:
            return False

    def set_filter(self, filter):
        # type: (Filter) -> None
        """Set the current active filter."""
        self._helper.set_filter(filter)
        self.cache.cache_post_change()

    def filter_cache_post_change(self):
        # type: () -> None
        """Called internally if the cache changes, emit a signal then."""
        self._helper.filter_cache_post_change(self.cache)

    def __getattr__(self, key):
        # type: (str) -> Any
        """we try to look exactly like a real cache."""
        return getattr(self.cache, key)


def cache_pre_changed(cache):
    # type: (Cache) -> None
    print("cache pre changed")


def cache_post_changed(cache):
    # type: (Cache) -> None
    print("cache post changed")


def _test():
    # type: () -> None
    """Internal test code."""
    print("Cache self test")
    apt_pkg.init()
    cache = Cache(apt.progress.text.OpProgress())
    cache.connect2("cache_pre_change", cache_pre_changed)
    cache.connect2("cache_post_change", cache_post_changed)
    print(("aptitude" in cache))
    pkg = cache["aptitude"]
    print(pkg.name)
    print(len(cache))

    for pkgname in cache.keys():
        assert cache[pkgname].name == pkgname

    cache.upgrade()
    changes = cache.get_changes()
    print(len(changes))
    for pkg in changes:
        assert pkg.name

    # see if fetching works
    for dirname in ["/tmp/pytest", "/tmp/pytest/partial"]:
        if not os.path.exists(dirname):
            os.mkdir(dirname)
    apt_pkg.config.set("Dir::Cache::Archives", "/tmp/pytest")
    pm = apt_pkg.PackageManager(cache._depcache)
    fetcher = apt_pkg.Acquire(apt.progress.text.AcquireProgress())
    cache._fetch_archives(fetcher, pm, None)
    #sys.exit(1)

    print("Testing filtered cache (argument is old cache)")
    filtered = FilteredCache(cache)
    filtered.cache.connect2("cache_pre_change", cache_pre_changed)
    filtered.cache.connect2("cache_post_change", cache_post_changed)
    filtered.cache.upgrade()
    filtered.set_filter(MarkedChangesFilter())
    print(len(filtered))
    for pkgname in filtered.keys():
        assert pkgname == filtered[pkgname].name

    print(len(filtered))

    print("Testing filtered cache (no argument)")
    filtered = FilteredCache(progress=apt.progress.base.OpProgress())
    filtered.cache.connect2("cache_pre_change", cache_pre_changed)
    filtered.cache.connect2("cache_post_change", cache_post_changed)
    filtered.cache.upgrade()
    filtered.set_filter(MarkedChangesFilter())
    print(len(filtered))
    for pkgname in filtered.keys():
        assert pkgname == filtered[pkgname].name

    print(len(filtered))


if __name__ == '__main__':
    _test()



#!/usr/bin/python3
"""Emulate dpkg --extract package.deb outdir"""

from __future__ import print_function

import os
import sys

import apt_inst


def main():
    """Main function."""
    if len(sys.argv) < 3:
        print("Usage: %s package.deb outdir\n" % (__file__), file=sys.stderr)
        sys.exit(1)
    if not os.path.exists(sys.argv[2]):
        print("The directory %s does not exist\n" % (sys.argv[2]),
              file=sys.stderr)
        sys.exit(1)

    fobj = open(sys.argv[1])
    try:
        apt_inst.DebFile(fobj).data.extractall(sys.argv[2])
    finally:
        fobj.close()


if __name__ == "__main__":
    main()



#!/usr/bin/python3
#  get_debian_mirrors.py - Parse Mirrors.masterlist and create a mirror list.
#
#  Copyright (c) 2010-2011 Julian Andres Klode <jak@debian.org>
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation; either version 2 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA
from __future__ import print_function
import collections
import sys
import urllib.request
from debian import deb822

mirrors = collections.defaultdict(set)
masterlist = urllib.request.urlopen("https://salsa.debian.org/mirror-team/"
                                    "masterlist/raw/master/"
                                    "Mirrors.masterlist")

for mirror in deb822.Deb822.iter_paragraphs(masterlist):
    if "Country" not in mirror:
        continue
    country = mirror["Country"].split(None, 1)[0]
    site = mirror["Site"]
    for proto in 'http', 'ftp':
        if "Archive-%s" % proto in mirror:
            mirrors[country].add("%s://%s%s" % (proto, site,
                                                mirror["Archive-%s" % proto]))

if len(mirrors) == 0:
    sys.stderr.write("E: Could not read the mirror list due to "
                     "some unknown issue\n")
    sys.exit(1)
for country in sorted(mirrors):
    print("#LOC:%s" % country)
    print("\n".join(sorted(mirrors[country])))



#!/usr/bin/python3

from __future__ import print_function

import contextlib
import errno
import itertools
import os
import shutil
import sys
import tempfile
import time
import unittest

if sys.version_info[0] > 2:
    from http.server import HTTPServer
    from http.server import SimpleHTTPRequestHandler as HTTPRequestHandler
    HTTPServer  # pyflakes
    HTTPRequestHandler  # pyflakes
else:
    from BaseHTTPServer import HTTPServer
    from SimpleHTTPServer import SimpleHTTPRequestHandler as HTTPRequestHandler


import apt_pkg
import apt.auth

import testcommon

WHEEZY_KEYID = "8B48AD6246925553"
WHEEZY_KEYDATE = "1335553717"
WHEEZY_KEY = """-----BEGIN PGP PUBLIC KEY BLOCK-----
Version: GnuPG v1.4.12 (GNU/Linux)


DHcut3Yey8o=
=id4q
-----END PGP PUBLIC KEY BLOCK-----"""


def normalize_key(keystr):
    """Remove the Version: header from a key block"""
    lines = keystr.split("\n")
    if lines[1].startswith("Version:"):
        return lines[:1] + lines[2:]
    return lines


class TestAuthKeys(testcommon.TestCase):

    """Test handling of keys for signed repositories."""

    def setUp(self):
        testcommon.TestCase.setUp(self)

        self.tmpdir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, self.tmpdir)
        apt_pkg.config.set("Dir", self.tmpdir)
        apt_pkg.config.set("Dir::Bin::Apt-key", "fakeroot-apt-key")
        apt_pkg.config.set("Dir::Etc", "etc/apt/")
        trustedparts_dir = apt_pkg.config.find_dir("Dir::Etc::Trustedparts")
        confparts_dir = apt_pkg.config.find_dir("Dir::Etc::parts")
        self.assertTrue(trustedparts_dir.startswith(self.tmpdir))
        os.makedirs(trustedparts_dir)
        os.makedirs(confparts_dir)
        shutil.copy("fakeroot-apt-key", self.tmpdir)

    @contextlib.contextmanager
    def _discard_stderr(self):
        stderr_fd = sys.stderr.fileno()
        stderr_save = os.dup(stderr_fd)
        try:
            devnull = os.open('/dev/null', os.O_WRONLY)
            try:
                os.dup2(devnull, stderr_fd)
                yield
            finally:
                os.close(devnull)
        finally:
            os.dup2(stderr_save, stderr_fd)
            os.close(stderr_save)

    def testAddAndExportKey(self):
        """Add an example key."""
        apt.auth.add_key(WHEEZY_KEY)
        # Strip the headers from the keys to avoid test errors because
        # the exported key used a differenct GnuPG version than the
        # original example key
        self.assertEqual(normalize_key(apt.auth.export_key(WHEEZY_KEYID)),
                         normalize_key(WHEEZY_KEY))

    def testAddAndListKey(self):
        """Add an example key and test if it is correctly returned by
        list_keys()
        """
        apt.auth.add_key(WHEEZY_KEY)
        ret = apt.auth.list_keys()
        self.assertEqual(len(ret), 1)
        key = ret[0]
        self.assertEqual(key.name,
                         "Debian Archive Automatic Signing Key (7.0/wheezy) "
                         "<ftpmaster@debian.org>")
        self.assertEqual(key.keyid, WHEEZY_KEYID)
        self.assertEqual(key.date, WHEEZY_KEYDATE)

    def testAddKeyFromFile(self):
        """Test adding a key from file."""
        keyfd, keyname = tempfile.mkstemp()
        self.addCleanup(os.close, keyfd)
        os.write(keyfd, WHEEZY_KEY.encode("UTF-8"))

        apt.auth.add_key_from_file(keyname)

        ret = apt.auth.list_keys()
        self.assertEqual(len(ret), 1)
        key = ret[0]
        self.assertEqual(key.name,
                         "Debian Archive Automatic Signing Key (7.0/wheezy) "
                         "<ftpmaster@debian.org>")
        self.assertEqual(key.keyid, WHEEZY_KEYID)
        self.assertEqual(key.date, WHEEZY_KEYDATE)

    def test_add_key_from_keyserver_too_short(self):
        """Ensure that short keyids are not imported"""
        with self.assertRaises(apt.auth.AptKeyIDTooShortError):
            apt.auth.add_key_from_keyserver(
                WHEEZY_KEYID, "hkp://localhost:19191")
        with self.assertRaises(apt.auth.AptKeyIDTooShortError):
            apt.auth.add_key_from_keyserver(
                "0101010178F7FE5C3E65D8AF8B48AD624692555",
                "hkp://localhost:19191")
        with self.assertRaises(apt.auth.AptKeyIDTooShortError):
            apt.auth.add_key_from_keyserver(
                "0x0101010178F7FE5C3E65D8AF8B48AD624692555",
                "hkp://localhost:19191")
        with self.assertRaises(apt.auth.AptKeyIDTooShortError):
            apt.auth.add_key_from_keyserver(
                "0101 0101 78F7 FE5C 3E65 D8AF 8B48 AD62 4692 555",
                "hkp://localhost:19191")

    def test_add_key_from_server_mitm(self):
        """Verify that the key fingerprint is verified after download"""
        self._start_keyserver()
        self.addCleanup(self._stop_keyserver)
        with self.assertRaises(apt.auth.AptKeyError) as cm:
            with self._discard_stderr():
                apt.auth.add_key_from_keyserver(
                    "0101010178F7FE5C3E65D8AF8B48AD6246925553",
                    "hkp://localhost:%d" % self.keyserver_port)
        self.assertTrue(
            str(cm.exception).startswith(
                "recv from 'hkp://localhost:%d' failed for '%s'" % (
                    self.keyserver_port,
                    "0101010178F7FE5C3E65D8AF8B48AD6246925553")),
            cm.exception)

    def testAddKeyFromServer(self):
        """Install a GnuPG key from a remote server."""
        self._start_keyserver()
        self.addCleanup(self._stop_keyserver)

        with self._discard_stderr():
            apt.auth.add_key_from_keyserver(
                "0xa1bD8E9D78F7FE5C3E65D8AF8B48AD6246925553",
                "hkp://localhost:%d" % self.keyserver_port)

        ret = apt.auth.list_keys()
        self.assertEqual(len(ret), 1)
        key = ret[0]
        self.assertEqual(key.name,
                         "Debian Archive Automatic Signing Key (7.0/wheezy) "
                         "<ftpmaster@debian.org>")
        self.assertEqual(key.keyid, WHEEZY_KEYID)
        self.assertEqual(key.date, WHEEZY_KEYDATE)

    def _start_keyserver(self):
        """Start a fake keyserver on http://localhost:19191
        If port 19191 is unavailable, try successive ports until one is.
        Store the port actually in use in self.keyserver_port.
        Thanks pitti.
        """
        dir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, dir)
        os.mkdir(os.path.join(dir, "pks"))
        with open(os.path.join(dir, "pks", "lookup"), "w") as key_file:
            key_file.write(WHEEZY_KEY)

        keyserver_pipe = os.pipe()
        self.keyserver_pid = os.fork()
        if self.keyserver_pid == 0:
            os.close(keyserver_pipe[0])
            # quiesce server log
            os.dup2(os.open('/dev/null', os.O_WRONLY), sys.stderr.fileno())
            os.chdir(dir)
            for port in itertools.count(19191):
                try:
                    httpd = HTTPServer(('localhost', port), HTTPRequestHandler)
                    break
                except IOError as e:
                    if e.errno != errno.EADDRINUSE:
                        raise
            keyserver_write = os.fdopen(keyserver_pipe[1], 'w')
            print(port, file=keyserver_write)
            keyserver_write.close()
            httpd.serve_forever()
            os._exit(0)

        os.close(keyserver_pipe[1])
        keyserver_read = os.fdopen(keyserver_pipe[0])
        self.keyserver_port = int(keyserver_read.readline())
        keyserver_read.close()

        # temporarily disable proxy, as gnupg does not get along with that
        # (LP #789049)
        self.orig_proxy = os.environ.get('http_proxy')
        try:
            del os.environ['http_proxy']
        except KeyError:
            pass

        # wait a bit until server is ready
        time.sleep(0.5)

    def _stop_keyserver(self):
        '''Stop fake keyserver'''
        assert self.keyserver_pid

        os.kill(self.keyserver_pid, 15)
        os.wait()

        # restore proxy
        if self.orig_proxy is not None:
            os.environ['http_proxy'] = self.orig_proxy


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2019 Colomban Wendling <cwendling@hypra.fr>
#
# Copying and distribution of this file, with or without modification,
# are permitted in any medium without royalty provided the copyright
# notice and this notice are preserved.
"""Helper checking argv[1] is a valid, writable, file descriptor"""

from sys import argv
import os

assert len(argv) == 2
with os.fdopen(int(argv[1]), 'w') as f:
    assert f.write('test') == 4



#!/usr/bin/python3

import apt_pkg

apt_pkg.init()

apt_pkg.config.set("APT::Acquire::Translation", "de")

cache = apt_pkg.Cache()
depcache = apt_pkg.DepCache(cache)

pkg = cache["gcc"]
cand = depcache.get_candidate_ver(pkg)
print(cand)

desc = cand.TranslatedDescription
print(desc)
print(desc.file_list)
(f, index) = desc.file_list.pop(0)

records = apt_pkg.PackageRecords(cache)
records.lookup((f, index))
desc = records.long_desc
print(len(desc))
print(desc)



#!/usr/bin/python3

import apt

cache = apt.Cache()

for pkg in cache:
    if not pkg.candidate.record:
        continue
    if "Task" in pkg.candidate.record:
        print("Pkg %s is part of '%s'" % (
            pkg.name, pkg.candidate.record["Task"].split()))
        #print pkg.candidateRecord



#!/usr/bin/python3
"""Emulate dpkg --info package.deb control-file"""

from __future__ import print_function

import sys

from apt_inst import DebFile


def main():
    """Main function."""
    if len(sys.argv) < 3:
        print('Usage: tool file.deb control-file\n', file=sys.stderr)
        sys.exit(0)
    fobj = open(sys.argv[1])
    try:
        print(DebFile(fobj).control.extractdata(sys.argv[2]))
    finally:
        fobj.close()


if __name__ == '__main__':
    main()



#!/usr/bin/python3

import sys
import unittest

import apt_inst

import testcommon

IS_NOT_32BIT = sys.maxsize > 2 ** 32


@unittest.skipIf(IS_NOT_32BIT, "Large File support is for 32 bit systems")
class testHashes(testcommon.TestCase):
    " test the hashsum functions against strings and files "

    LARGE_PACKAGE_CONTENT = "data/test_debs/large-package-content_1.0_all.deb"

    def testExtractData(self):
        deb = apt_inst.DebFile(self.LARGE_PACKAGE_CONTENT)

        self.assertRaises(MemoryError, deb.data.extractdata, "large-file")


if __name__ == "__main__":
    unittest.main()



# apt/progress/__init__.py - Initialization file for apt.progress.
#
# Copyright (c) 2009 Julian Andres Klode <jak@debian.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
"""Progress reporting.

This package provides progress reporting for the python-apt package. The module
'base' provides classes with no output, and the module 'text' provides classes
for terminals, etc.
"""

from __future__ import print_function

try:
    from typing import Sequence
    Sequence  # pylint
except ImportError:
    pass


__all__ = []  # type: Sequence[str]



#!/usr/bin/python3
import apt_pkg


def main():
    """Example for PackageFile()"""
    apt_pkg.init()
    cache = apt_pkg.Cache()
    for pkgfile in cache.file_list:
        print('Package-File:', pkgfile.filename)
        print('Index-Type:', pkgfile.index_type)  # 'Debian Package Index'
        if pkgfile.not_source:
            print('Source: None')
        else:
            if pkgfile.site:
                # There is a source, and a site, print the site
                print('Source:', pkgfile.site)
            else:
                # It seems to be a local repository
                print('Source: Local package file')
        if pkgfile.not_automatic:
            # The system won't be updated automatically (eg. experimental)
            print('Automatic: No')
        else:
            print('Automatic: Yes')
        print()


if __name__ == '__main__':
    main()



#!/usr/bin/python3

# This is a simple clone of tests/versiontest.cc
import apt_pkg
import sys
import re

apt_pkg.init_config()
apt_pkg.init_system()

TestFile = apt_pkg.parse_commandline(apt_pkg.config, [], sys.argv)
if len(TestFile) != 1:
    print("Must have exactly 1 file name")
    sys.exit(0)

# Go over the file..
list = open(TestFile[0], "r")
CurLine = 0
while(1):
    Line = list.readline()
    CurLine = CurLine + 1
    if Line == "":
        break
    Line = Line.strip()
    if len(Line) == 0 or Line[0] == '#':
        continue

    Split = re.split("[ \n]", Line)

    # Check forward
    if apt_pkg.version_compare(Split[0], Split[1]) != int(Split[2]):
        print("Comparision failed on line %u. '%s' ? '%s' %i != %i" % (CurLine,
            Split[0], Split[1], apt_pkg.version_compare(Split[0], Split[1]),
            int(Split[2])))
    # Check reverse
    if apt_pkg.version_compare(Split[1], Split[0]) != -1 * int(Split[2]):
        print("Comparision failed on line %u. '%s' ? '%s' %i != %i" % (CurLine,
            Split[1], Split[0], apt_pkg.version_compare(Split[1], Split[0]),
            -1 * int(Split[2])))



#!/usr/bin/python3
import sys

import apt_pkg
import apt


def show_help():
    print("apt %s compiled on %s %s" % (apt_pkg.VERSION,
                                        apt_pkg.DATE, apt_pkg.TIME))
    if apt_pkg.config.find_b("version"):
        return 0

    # Copied from apt-cdrom
    print("Usage: apt-cdrom [options] command\n"
          "\n"
          "apt-cdrom is a tool to add CDROM's to APT's source list. The\n"
          "CDROM mount point and device information is taken from apt.conf\n"
          "and /etc/fstab.\n"
          "\n"
          "Commands:\n"
          "   add - Add a CDROM\n"
          "   ident - Report the identity of a CDROM\n"
          "\n"
          "Options:\n"
          "  -h   This help text\n"
          "  -d   CD-ROM mount point\n"
          "  -r   Rename a recognized CD-ROM\n"
          "  -m   No mounting\n"
          "  -f   Fast mode, don't check package files\n"
          "  -a   Thorough scan mode\n"
          "  -c=? Read this configuration file\n"
          "  -o=? Set an arbitrary configuration option, eg -o "
          "dir::cache=/tmp\n"
          "See fstab(5)")
    return 0


def main(args):
    arguments = apt_pkg.parse_commandline(apt_pkg.config,
                    [('h', "help", "help"),
                     ('v', "version", "version"),
                     ('d', "cdrom", "Acquire::cdrom::mount", "HasArg"),
                     ('r', "rename", "APT::CDROM::Rename"),
                     ('m', "no-mount", "APT::CDROM::NoMount"),
                     ('f', "fast", "APT::CDROM::Fast"),
                     ('n', "just-print", "APT::CDROM::NoAct"),
                     ('n', "recon", "APT::CDROM::NoAct"),
                     ('n', "no-act", "APT::CDROM::NoAct"),
                     ('a', "thorough", "APT::CDROM::Thorough"),
                     ('c', "config-file", "", "ConfigFile"),
                     ('o', "option", "", "ArbItem")], args)

    if apt_pkg.config.find_b("help") or apt_pkg.config.find_b("version"):
        return show_help()

    progress = apt.progress.text.CdromProgress()
    cdrom = apt_pkg.Cdrom()

    if not arguments:
        return show_help()
    elif arguments[0] == 'add':
        cdrom.add(progress)
    elif arguments[0] == 'ident':
        cdrom.ident(progress)
    else:
        sys.stderr.write('E: Invalid operation %s\n' % arguments[0])
        return 1


if __name__ == '__main__':
    sys.exit(main(sys.argv))



#!/usr/bin/python3

import apt_pkg

apt_pkg.init()

sources = apt_pkg.SourceList()
sources.read_main_list()

cache = apt_pkg.Cache()
depcache = apt_pkg.DepCache(cache)
pkg = cache["libimlib2"]
cand = depcache.get_candidate_ver(pkg)
for (f, i) in cand.file_list:
    index = sources.find_index(f)
    print(index)
    if index:
        print(index.size)
        print(index.is_trusted)
        print(index.exists)
        print(index.Haspackages)
        print(index.archive_uri("some/path"))



#!/usr/bin/python3
# example how to deal with the depcache

import apt_pkg
import sys

from progress import TextCdromProgress


# init
apt_pkg.init()

cdrom = apt_pkg.Cdrom()
print(cdrom)

progress = TextCdromProgress()

(res, ident) = cdrom.ident(progress)
print("ident result is: %s (%s) " % (res, ident))

apt_pkg.config["APT::CDROM::Rename"] = "True"
cdrom.add(progress)

print("Exiting")
sys.exit(0)



#!/usr/bin/python3
# -*- Mode: Python; indent-tabs-mode: nil; tab-width: 4; coding: utf-8 -*-

import os
import subprocess
import unittest

import testcommon


class TestPyflakesClean(testcommon.TestCase):

    EXCLUDES = ["build", "tests/old", ".pybuild"]
    TOPLEVEL = os.path.normpath(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), ".."))

    def is_excluded_path(self, path):
        for exclude in self.EXCLUDES:
            if path.startswith(os.path.join(self.TOPLEVEL, exclude)):
                return True
        return False

    def get_py_files(self, toplevel):
        files = []
        for path, dirnames, filenames in os.walk(self.TOPLEVEL):
            if self.is_excluded_path(path):
                continue
            for filename in filenames:
                if os.path.splitext(filename)[1] == ".py":
                    files.append(os.path.join(path, filename))
        return files

    def test_pyflakes_clean(self):
        cmd = ["pyflakes"] + self.get_py_files(self.TOPLEVEL)
        res = subprocess.call(cmd)
        if res != 0:
            self.fail("pyflakes failed with: %s" % res)


if __name__ == "__main__":
    import logging
    logging.basicConfig(level=logging.DEBUG)
    unittest.main()



#!/usr/bin/python3
# Builds on python2.X and python3
# $Id: setup.py,v 1.2 2002/01/08 07:13:21 jgg Exp $
import glob
import os
import shutil
import sys

from setuptools import setup, Extension
from setuptools.command.install import install
cmdclass = {}

try:
    from DistUtilsExtra.command import build_extra, build_i18n
    from DistUtilsExtra.auto import clean_build_tree
except ImportError:
    print('W: [python%s] DistUtilsExtra import error.' % sys.version[:3])
else:
    cmdclass['build'] = build_extra.build_extra
    cmdclass['build_i18n'] = build_i18n.build_i18n
    cmdclass['clean'] = clean_build_tree

try:
    from sphinx.setup_command import BuildDoc
except ImportError:
    print('W: [python%s] Sphinx import error.' % sys.version[:3])
else:
    cmdclass['build_sphinx'] = BuildDoc


class InstallTypeinfo(install):
    def run(self):
        install.run(self)
        for pyi in glob.glob("typehinting/*.pyi"):
            stubs = os.path.basename(pyi).split(".")[0] + "-stubs"
            stubs = os.path.join(self.install_purelib, stubs)
            if not os.path.exists(stubs):
                os.makedirs(stubs)
            shutil.copy(pyi, os.path.join(stubs, "__init__.pyi"))


cmdclass['install'] = InstallTypeinfo


def get_version():
    """Get a PEP 0440 compatible version string"""
    version = os.environ.get('DEBVER')
    if not version:
        return version

    version = version.replace("~alpha", ".a")
    version = version.replace("~beta", ".b")
    version = version.replace("~rc", ".rc")
    version = version.replace("~exp", ".dev")
    version = version.replace("ubuntu", "+ubuntu")
    version = version.replace("tanglu", "+tanglu")
    version = version.split("build")[0]

    return version


# The apt_pkg module.
files = ['apt_pkgmodule.cc', 'acquire.cc', 'cache.cc', 'cdrom.cc',
         'configuration.cc', 'depcache.cc', 'generic.cc', 'hashes.cc',
         'hashstring.cc', 'indexfile.cc', 'metaindex.cc',
         'pkgmanager.cc', 'pkgrecords.cc', 'pkgsrcrecords.cc', 'policy.cc',
         'progress.cc', 'sourcelist.cc', 'string.cc', 'tag.cc',
         'lock.cc', 'acquire-item.cc', 'python-apt-helpers.cc',
         'cachegroup.cc', 'orderlist.cc', 'hashstringlist.cc']
files = sorted(['python/' + fname for fname in files], key=lambda s: s[:-3])
apt_pkg = Extension("apt_pkg", files, libraries=["apt-pkg"],
                    extra_compile_args=['-std=c++11', '-Wno-write-strings',
                                        '-DAPT_8_CLEANER_HEADERS',
                                        '-DAPT_9_CLEANER_HEADERS',
                                        '-DAPT_10_CLEANER_HEADERS',
                                        '-DPY_SSIZE_T_CLEAN'])

# The apt_inst module
files = ["python/apt_instmodule.cc", "python/generic.cc",
         "python/arfile.cc", "python/tarfile.cc"]
apt_inst = Extension("apt_inst", files, libraries=["apt-pkg"],
                     extra_compile_args=['-std=c++11', '-Wno-write-strings',
                                         '-DPY_SSIZE_T_CLEAN'])

# Replace the leading _ that is used in the templates for translation
if len(sys.argv) > 1 and sys.argv[1] == "build":
    if not os.path.exists("build/data/templates/"):
        os.makedirs("build/data/templates")
    for template in glob.glob('data/templates/*.info.in'):
        source = open(template, "r")
        build = open("build/" + template[:-3], "w")
        for line in source:
            build.write(line.lstrip("_"))
        source.close()
        build.close()
    for template in glob.glob('data/templates/*.mirrors'):
        shutil.copy(template, os.path.join("build", template))


setup(name="python-apt",
      description="Python bindings for APT",
      version=get_version(),
      author="APT Development Team",
      author_email="deity@lists.debian.org",
      ext_modules=[apt_pkg, apt_inst],
      packages=['apt', 'apt.progress', 'aptsources'],
      package_data={
          'apt': ["*.pyi", "py.typed"],
      },
      data_files=[('share/python-apt/templates',
                   glob.glob('build/data/templates/*.info')),
                  ('share/python-apt/templates',
                   glob.glob('data/templates/*.mirrors'))],
      cmdclass=cmdclass,
      license='GNU GPL',
      platforms='posix',
      )



from typing import *

from apt.progress.base import (
    AcquireProgress,
    CdromProgress,
    InstallProgress,
    OpProgress,
)

class FileLike(Protocol):
    def fileno(self) -> int: ...

class Cdrom:
    def add(self, progress: CdromProgress) -> bool: ...
    def ident(self, progress: CdromProgress) -> str: ...

@overload
def gettext(msg: str, domain: str) -> str: ...
@overload
def gettext(msg: str) -> str: ...

class Configuration(Mapping[str, str]):
    def find_file(self, key: str, default: str="") -> str: ...
    def find_dir(self, key: str, default: str="") -> str: ...
    def dump(self) -> str: ...
    def find(self, key: str, default: object=None) -> str: ...
    def find_b(self, key: str, default: bool=False) -> bool: ...
    def set(self, key: str, value: str) -> None: ...
    def value_list(self, key: str) -> List[str]: ...
    def clear(self, root: object=None) -> None: ...
    def __getitem__(self, key: str) -> str: ...
    def __iter__(self) -> Iterator[str]: ...
    def __len__(self) -> int: ...

config = Configuration()

def init() -> None: ...
def init_config() -> None: ...
def init_system() -> None: ...

# FIXME: this is really a file-like object
def md5sum(o: Any) -> str: ...

class Dependency():
    comp_type: str
    comp_type_deb: str
    target_pkg: Package
    target_ver: str
    dep_type_untranslated: str
    def all_targets(self) -> List[Version]: ...

# This is really a SystemError, but we don't want to expose that
class Error(Exception):
    pass

class Package():
    name: str
    version_list: List[Version]
    architecture: str
    id: int
    current_ver: Version
    essential: bool
    current_state: int
    inst_state: int
    selected_state: int
    has_versions: bool
    has_provides: bool
    provides_list: List[Tuple[str, str, Version]]
    def get_fullname(self, pretty: bool=False) -> str: ...

class ProblemResolver:
    def __init__(self, cache: DepCache) -> None: ...
    def clear(self, pkg: Package) -> None: ...
    def protect(self, pkg: Package) -> None: ...
    def remove(self, pkg: Package) -> None: ...
    def resolve(self, fix_broken: bool=True) -> bool: ...
    def resolve_by_keep(self) -> bool: ...

CURSTATE_CONFIG_FILES: int
INSTSTATE_REINSTREQ: int
INSTSTATE_HOLD_REINSTREQ: int
    
class Version():
    ver_str: str
    hash: int
    file_list: List[Tuple[PackageFile, int]]
    translated_description: Description
    installed_size: int
    size: int
    arch: str
    downloadable: bool
    id: int
    section: str
    priority: int
    priority_str: str
    provides_list: List[Tuple[str,str,str]]
    depends_list: Dict[str, List[List[Dependency]]]
    parent_pkg: Package
    multi_arch: int
    MULTI_ARCH_ALL: int
    MULTI_ARCH_ALLOWED: int
    MULTI_ARCH_ALL_ALLOWED: int
    MULTI_ARCH_ALL_FOREIGN: int
    MULTI_ARCH_FOREIGN: int
    MULTI_ARCH_NO: int
    MULTI_ARCH_NONE: int
    MULTI_ARCH_SAME: int

class Description():
    file_list: List[Tuple[PackageFile, int]]

class PackageRecords():
    homepage: str
    short_desc: str
    long_desc: str
    source_pkg: str
    source_ver: str
    record: str
    filename: str
    md5_hash: str
    sha1_hash: str
    sha256_hash: str
    hashes: HashStringList
    def __init__(self, cache: Cache) -> None: ...
    def lookup(self, packagefile: Tuple[PackageFile, int], index: int=0) -> bool: ...

class PackageFile:
    architecture: str
    archive: str
    codename: str
    component: str
    filename: str
    id: int
    index_type: str
    label: str
    not_automatic: bool
    not_source: bool
    origin: str
    site: str
    size: int
    version: str

class TagFile(Iterator[TagSection[AnyStr]]):
    @overload
    def __new__(cls, file: object) -> TagFile[str]: ...
    @overload
    def __new__(cls, file: object, bytes: Literal[True]) -> TagFile[bytes]: ...
    @overload
    def __new__(cls, file: object, bytes: Literal[False]) -> TagFile[str]: ...
    def __iter__(self) -> Iterator[TagSection[AnyStr]]: ...
    def __next__(self) -> TagSection[AnyStr]: ...

class TagSection(Mapping[str, AnyStr]):
    @overload
    def __new__(cls, str: Union[str, bytes]) -> TagSection[str]: ...
    @overload
    def __new__(cls, str: Union[str, bytes], bytes: Literal[True]) -> TagSection[bytes]: ...
    @overload
    def __new__(cls, str: Union[str, bytes], bytes: Literal[False]) -> TagSection[str]: ...
    def __getitem__(self, key: str) -> AnyStr: ...
    def get(self, key: str, default: Optional[object] = None) -> AnyStr: ...
    def find(self, key: str, default: Optional[object] = None) -> AnyStr: ...
    def find_raw(self, key: str, default: Optional[object] = None) -> AnyStr: ...
    def __contains__(self, key: object) -> bool: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[str]: ...

def version_compare(a: str, b: str) -> int: ...

def get_lock(file: str, errors: bool=False) -> int: ...
def pkgsystem_lock() -> None: ...
def pkgsystem_unlock() -> None: ...
def read_config_file(configuration: Configuration, path: str) -> None: ...
def read_config_dir(configuration: Configuration, path: str) -> None: ...

def pkgsystem_lock_inner() -> None: ...
def pkgsystem_unlock_inner() -> None: ...
def pkgsystem_is_locked() -> bool: ...

SELSTATE_HOLD: int

class Acquire:
    fetch_needed: int
    items: List[AcquireItem]
    partial_present: int
    total_needed: int
    workers: List[AcquireWorker]
    RESULT_CANCELLED: int
    RESULT_FAILED: int
    RESULT_CONTINUE: int
    def __init__(self, progress: Optional[AcquireProgress]=None) -> None: ...
    def run(self) -> int: ...
    def shutdown(self) -> None: ...
    def get_lock(self, path: str) -> None: ...

class AcquireWorker:
    current_item: AcquireItemDesc
    current_size: int
    total_size: int
    status: str
    
class AcquireItem:
    active_subprocess: str
    complete: bool
    desc_uri: str
    destfile: str
    error_text: str
    filesize: int
    id: int
    is_trusted: bool
    local: bool
    mode: str
    partialsize: int
    status: int
    
    STAT_IDLE: int
    STAT_FETCHING: int
    STAT_DONE: int
    STAT_ERROR: int
    STAT_AUTH_ERROR: int
    STAT_TRANSIENT_NETWORK_ERROR: int

class AcquireItemDesc:
    description: str
    owner: AcquireItem
    shortdesc: str
    uri: str

class Hashes:
    def __init__(self, object: Union[bytes, FileLike, int]) -> None: ...
    hashes: HashStringList

class HashString:
    def __init__(self, type: str, hash: Optional[str] = None) -> None: ...
    def verify_file(self, filename: str) -> bool: ...

    hashtype: str
    hashvalue: str
    usable: bool

class HashStringList:
    def append(self, object: HashString) -> None: ...
    def find(self, type: str = "") -> HashString: ...
    def verify_file(self, filename: str) -> bool: ...

    file_size: int
    usable: bool

class AcquireFile(AcquireItem):
    def __init__(self, owner: Acquire, uri: str, hash: Optional[Union[HashStringList, str]], size: int=0, descr: str="", short_descr: str="", destdir: str="", destfile: str="") -> None: ...

class IndexFile:
    def archive_uri(self, path: str) -> str: ...
    describe: str
    exists: bool
    has_packages: bool
    is_trusted: bool
    label: str
    size: int
    
class SourceRecordFiles:

    hashes: HashStringList
    path: str
    size: int
    type: str

class SourceRecords:
    def lookup(self, name: str) -> bool: ...
    def restart(self) -> None: ...
    def step(self) -> bool: ...
    binaries: List[str]
    version: str
    files: List[SourceRecordFiles]
    index: IndexFile
    package: str
    section: str

class ActionGroup:
    def __init__(self, depcache: DepCache) -> None: ...

class MetaIndex:
    dist: str
    index_files: List[IndexFile]
    is_trusted: bool
    uri: str

class SourceList():
    list: List[MetaIndex]
    def read_main_list(self) -> None: ...
    def find_index(self, pf: PackageFile) -> IndexFile: ...

class PackageManager():
    RESULT_FAILED: int
    RESULT_COMPLETED: int
    RESULT_INCOMPLETE: int
    def __init__(self, depcache: DepCache) -> None: ...
    def get_archives(self, fetcher: Acquire, list: SourceList, recs: PackageRecords) -> bool: ...

class Cache():
    packages: List[Package]
    def __init__(self, progress: Optional[OpProgress]=None) -> None: ...
    def __contains__(self, name: Union[str, Tuple[str, str]]) -> Package: ...
    def __getitem__(self, name: Union[str, Tuple[str, str]]) -> Package: ...
    def __len__(self) -> int: ...
    def update(self, progress: AcquireProgress, sources: SourceList, pulse_interval: int) -> int: ...
    
class DepCache():
    broken_count: int
    inst_count: int
    del_count: int
    keep_count: int
    usr_size: int
    policy: Policy
    def __init__(self, cache: Cache) -> None: ...
    def init(self, progress: Optional[OpProgress]=None) -> None: ...
    def get_candidate_ver(self, pkg: Package) -> Optional[Version]: ...
    def set_candidate_ver(self, pkg: Package, ver: Version) -> bool: ...
    def marked_install(self, pkg: Package) -> bool: ...
    def marked_upgrade(self, pkg: Package) -> bool: ...
    def marked_keep(self, pkg: Package) -> bool: ...
    def marked_downgrade(self, pkg: Package) -> bool: ...
    def marked_delete(self, pkg: Package) -> bool: ...
    def marked_reinstall(self, pkg: Package) -> bool: ...

    def is_upgradable(self, pkg: Package) -> bool: ...
    def is_garbage(self, pkg: Package) -> bool: ...
    def is_auto_installed(self, pkg: Package) -> bool: ...
    def is_inst_broken(self, pkg: Package) -> bool: ...
    def is_now_broken(self, pkg: Package) -> bool: ...

    def mark_keep(self, pkg: Package) -> None: ...
    def mark_install(self, pkg: Package, auto_inst: bool=True, from_user: bool=True) -> None: ...
    def mark_delete(self, pkg: Package, purge: bool=False) -> None: ...
    def mark_auto(self, pkg: Package, auto: bool) -> None: ...
    def commit(self, acquire_progress: AcquireProgress, install_progress: InstallProgress) -> None: ...

    def upgrade(self, dist_upgrade: bool=True) -> bool: ...
    def fix_broken(self) -> bool: ...

class Policy():
    def get_priority(self, pkg: Union[PackageFile, Version]) -> int: ...

class SystemLock():
    def __enter__(self) -> None: ...
    def __exit__(self, typ: object, value: object, traceback: object) -> None: ...
    
class FileLock():
    def __init__(self, path: str) -> None: ...
    def __enter__(self) -> None: ...
    def __exit__(self, typ: object, value: object, traceback: object) -> None: ...

def upstream_version(ver: str) -> str: ...
def get_architectures() -> List[str]: ...    
def check_dep(pkg_ver: str, dep_op: str, dep_ver: str) -> bool: ...
def uri_to_filename(uri: str) -> str: ...
def str_to_time(rfc_time: str) -> int: ...
def time_to_str(time: int) -> str: ...
def size_to_str(size: Union[float, int]) -> str: ...
def open_maybe_clear_signed_file(file: str) -> int: ...

def parse_depends(s: str, strip_multi_arch: bool = True, architecture: str = '') -> List[List[Tuple[str, str, str]]]: ...
def parse_src_depends(s: str, strip_multi_arch: bool = True, architecture: str = '') -> List[List[Tuple[str, str, str]]]: ...



import unittest

import apt_pkg

import testcommon


class TestGroup(testcommon.TestCase):

    def setUp(self):
        testcommon.TestCase.setUp(self)
        self.cache = apt_pkg.Cache(progress=None)

    def test_pkgingroup(self):
        """Check that each package belongs to the corresponding group"""
        for pkg in self.cache.packages:
            group = apt_pkg.Group(self.cache, pkg.name)
            assert any(pkg.id == p.id for p in group)

    def test_iteration(self):
        """Check that iteration works correctly."""
        for pkg in self.cache.packages:
            group = apt_pkg.Group(self.cache, pkg.name)

            list(group) == list(group)

    def test_cache_groups(self):
        """group: Iterate over all groups"""
        assert len(list(self.cache.groups)) == self.cache.group_count


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3
import apt_pkg

Parse = apt_pkg.TagFile(open("/var/lib/dpkg/status", "r"))

while Parse.step() == 1:
    print(Parse.section.get("Package"))
    print(apt_pkg.parse_depends(Parse.section.get("Depends", "")))



#!/usr/bin/python3

import apt_pkg

apt_pkg.init()

sources = apt_pkg.SourceList()
sources.read_main_list()


for metaindex in sources.list:
    print(metaindex)
    print("uri: ", metaindex.uri)
    print("dist: ", metaindex.dist)
    print("index_files: ", "\n".join([str(i) for i in metaindex.index_files]))
    print()



"""Common testing stuff"""

import apt_pkg
import os

import sys
import unittest


class TestCase(unittest.TestCase):
    """Base class for python-apt unittests"""

    def setUp(self):
        self.resetConfig()

    def resetConfig(self):
        apt_pkg.config.clear("")
        for key in apt_pkg.config.list():
            apt_pkg.config.clear(key)

        # Avoid loading any host config files
        os.unsetenv("APT_CONFIG")
        apt_pkg.config["Dir::Etc::main"] = "/dev/null"
        apt_pkg.config["Dir::Etc::parts"] = "/dev/null"

        apt_pkg.init_config()
        apt_pkg.init_system()

        # Restore default values
        apt_pkg.config["Dir::Etc::main"] = "apt.conf"
        apt_pkg.config["Dir::Etc::parts"] = "apt.conf.d"

    if (sys.version_info.major, sys.version_info.minor) < (3, 2):
        def assertRaisesRegex(self, exc_typ, regex, func, *args, **kwds):
            """Compatibility helper"""
            try:
                func(*args, **kwds)
            except Exception as exc:
                self.assertIsInstance(exc, exc_typ)
                self.assertRegexpMatches(str(exc), regex)
            else:
                self.fail("Did not raise exception")



#!/usr/bin/python3
# some example for apt_inst

import apt_pkg
import apt_inst
import sys
import os.path


def Callback(member, data):
    """ callback for debExtract """
    print("'%s','%s',%u,%u,%u,%u,%u,%u,%u"
          % (member.name, member.linkname, member.mode, member.uid,
             member.gid, member.size, member.mtime, member.major,
             member.minor))


if __name__ == "__main__":
    if len(sys.argv) < 2:
        print("need filename argumnet")
        sys.exit(1)
    file = sys.argv[1]

    print("Working on: %s" % file)
    print("Displaying data.tar.gz:")
    apt_inst.DebFile(open(file)).data.go(Callback)

    print("Now extracting the control file:")
    control = apt_inst.DebFile(open(file)).control.extractdata("control")
    sections = apt_pkg.TagSection(control)

    print("Maintainer is: ")
    print(sections["Maintainer"])

    print()
    print("DependsOn: ")
    depends = sections["Depends"]
    print(apt_pkg.parse_depends(depends))

    print("extracting archive")
    dir = "/tmp/deb"
    os.mkdir(dir)
    apt_inst.DebFile(open(file)).data.extractall(dir)

    def visit(arg, dirname, names):
        print("%s/" % dirname)
        for file in names:
            print("\t%s" % file)

    os.path.walk(dir, visit, None)



#!/usr/bin/python3
#
# Copyright (C) 2009 Julian Andres Klode <jak@debian.org>
#
# Copying and distribution of this file, with or without modification,
# are permitted in any medium without royalty provided the copyright
# notice and this notice are preserved.
"""Unit tests for verifying the correctness of hashsums in apt_pkg.

Unit tests to verify the correctness of Hashes, HashString and the various
functions like md5sum."""
import unittest
import hashlib
import sys
import warnings

import apt_pkg

import testcommon


class TestHashes(testcommon.TestCase):
    """Test apt_pkg.Hashes() and the various apt_pkg.*sum() functions."""

    def setUp(self):
        """Prepare the tests, create reference values..."""
        testcommon.TestCase.setUp(self)
        self.file = open(apt_pkg.__file__, "rb")
        self.value = self.file.read()
        self.hashes = apt_pkg.Hashes(self.value)
        self.file.seek(0)
        self.fhashes = apt_pkg.Hashes(self.file)
        # Reference values.
        self.md5 = hashlib.md5(self.value).hexdigest()
        self.sha1 = hashlib.sha1(self.value).hexdigest()
        self.sha256 = hashlib.sha256(self.value).hexdigest()
        self.file.seek(0)

        warnings.filterwarnings("ignore", category=DeprecationWarning)

    def tearDown(self):
        """Cleanup, Close the file object used for the tests."""
        testcommon.TestCase.tearDown(self)
        warnings.resetwarnings()
        self.file.close()

    def test_md5sum(self):
        """hashes: Test apt_pkg.md5sum()"""
        self.assertEqual(apt_pkg.md5sum(self.value), self.md5)
        self.assertEqual(apt_pkg.md5sum(self.file), self.md5)

    def test_sha1sum(self):
        """hashes: Test apt_pkg.sha1sum()"""
        self.assertEqual(apt_pkg.sha1sum(self.value), self.sha1)
        self.assertEqual(apt_pkg.sha1sum(self.file), self.sha1)

    def test_sha256sum(self):
        """hashes: Test apt_pkg.sha256sum()"""
        self.assertEqual(apt_pkg.sha256sum(self.value), self.sha256)
        self.assertEqual(apt_pkg.sha256sum(self.file), self.sha256)

    def test_bytes(self):
        """hashes: Test apt_pkg.Hashes(bytes)"""
        self.assertEqual(self.hashes.hashes.find("md5sum").hashvalue,
                         self.md5)
        self.assertEqual(self.hashes.hashes.find("md5sum"),
                         apt_pkg.HashString("MD5Sum", self.md5))
        self.assertEqual(self.hashes.hashes.find("sha1"),
                         apt_pkg.HashString("SHA1", self.sha1))
        self.assertEqual(self.hashes.hashes.find("sha256"),
                         apt_pkg.HashString("SHA256", self.sha256))
        self.assertRaises(KeyError, self.hashes.hashes.find, "md5")

    def test_file(self):
        """hashes: Test apt_pkg.Hashes(file)."""
        self.assertEqual(self.fhashes.hashes.find("md5sum").hashvalue,
                         self.md5)
        self.assertEqual(self.fhashes.hashes.find("md5sum"),
                         apt_pkg.HashString("MD5Sum", self.md5))
        self.assertEqual(self.fhashes.hashes.find("sha1"),
                         apt_pkg.HashString("SHA1", self.sha1))
        self.assertEqual(self.fhashes.hashes.find("sha256"),
                         apt_pkg.HashString("SHA256", self.sha256))

    def test_unicode(self):
        """hashes: Test apt_pkg.Hashes(unicode)."""
        if sys.version_info[0] == 3:
            self.assertRaises(TypeError, apt_pkg.Hashes, "D")
            self.assertRaises(TypeError, apt_pkg.md5sum, "D")
            self.assertRaises(TypeError, apt_pkg.sha1sum, "D")
            self.assertRaises(TypeError, apt_pkg.sha256sum, "D")
        else:
            self.assertRaises(TypeError, apt_pkg.Hashes, unicode())
            self.assertRaises(TypeError, apt_pkg.md5sum, unicode())
            self.assertRaises(TypeError, apt_pkg.sha1sum, unicode())
            self.assertRaises(TypeError, apt_pkg.sha256sum, unicode())


class TestHashString(testcommon.TestCase):
    """Test apt_pkg.HashString()."""

    def setUp(self):
        """Prepare the test by reading the file."""
        testcommon.TestCase.setUp(self)
        self.file = open(apt_pkg.__file__)
        self.hashes = apt_pkg.Hashes(self.file)

        self.md5 = self.hashes.hashes.find("md5sum")
        self.sha1 = self.hashes.hashes.find("sha1")
        self.sha256 = self.hashes.hashes.find("sha256")

    def tearDown(self):
        """Cleanup, Close the file object used for the tests."""
        self.file.close()

    def test_md5(self):
        """hashes: Test apt_pkg.HashString().md5"""
        self.assertIn("MD5Sum:", str(self.md5))
        self.assertTrue(self.md5.verify_file(apt_pkg.__file__))

    def test_sha1(self):
        """hashes: Test apt_pkg.HashString().sha1"""
        self.assertIn("SHA1:", str(self.sha1))
        self.assertTrue(self.sha1.verify_file(apt_pkg.__file__))

    def test_sha256(self):
        """hashes: Test apt_pkg.HashString().sha256"""
        self.assertIn("SHA256:", str(self.sha256))
        self.assertTrue(self.sha256.verify_file(apt_pkg.__file__))

    def test_wrong(self):
        """hashes: Test apt_pkg.HashString(wrong_type)."""
        self.assertRaises(TypeError, apt_pkg.HashString, 0)
        if sys.version_info[0] == 3:
            self.assertRaises(TypeError, apt_pkg.HashString, bytes())


class TestHashStringList(testcommon.TestCase):
    """Test apt_pkg.HashStringList()"""

    def test_file_size(self):
        hsl = apt_pkg.HashStringList()
        self.assertEqual(hsl.file_size, 0)
        hsl.file_size = 42
        self.assertEqual(hsl.file_size, 42)
        self.assertEqual(len(hsl), 1)

        # Verify that I can re-assign value (this handles the long case on
        # Python 2).
        hsl.file_size = hsl.file_size

        with self.assertRaises(OverflowError):
            hsl.file_size = -1

        hsl.file_size = 0

    def test_append(self):
        """Testing whether append works correctly."""
        hs1 = apt_pkg.HashString("MD5Sum",
                                 "a60599e6200b60050d7a30721e3532ed")
        hs2 = apt_pkg.HashString("SHA1",
                                 "ef113338e654b1ada807a939ad47b3a67633391b")

        hsl = apt_pkg.HashStringList()
        hsl.append(hs1)
        hsl.append(hs2)
        self.assertEqual(len(hsl), 2)
        self.assertEqual(hsl[0].hashtype, "MD5Sum")
        self.assertEqual(hsl[1].hashtype, "SHA1")
        self.assertEqual(str(hsl[0]), str(hs1))
        self.assertEqual(str(hsl[1]), str(hs2))

    def test_find(self):
        """Testing whether append works correctly."""
        hs1 = apt_pkg.HashString("MD5Sum",
                                 "a60599e6200b60050d7a30721e3532ed")
        hs2 = apt_pkg.HashString("SHA1",
                                 "ef113338e654b1ada807a939ad47b3a67633391b")

        hsl = apt_pkg.HashStringList()
        hsl.append(hs1)
        hsl.append(hs2)

        self.assertEqual(hsl.find("MD5Sum").hashtype, "MD5Sum")
        self.assertEqual(hsl.find("SHA1").hashtype, "SHA1")
        self.assertEqual(hsl.find().hashtype, "SHA1")

    def test_verify_file(self):
        with open(apt_pkg.__file__) as fobj:
            hashes = apt_pkg.Hashes(fobj)
            with warnings.catch_warnings(record=True):
                warnings.simplefilter("always")
                sha1 = hashes.hashes.find("sha1")
                sha256 = hashes.hashes.find("sha256")

        hsl = apt_pkg.HashStringList()
        hsl.append(sha1)
        hsl.append(sha256)

        self.assertTrue(hsl.verify_file(apt_pkg.__file__))

        md5sum = apt_pkg.HashString("MD5Sum",
                                    "a60599e6200b60050d7a30721e3532ed")
        hsl.append(md5sum)

        self.assertFalse(hsl.verify_file(apt_pkg.__file__))

        hsl2 = hashes.hashes
        self.assertIsInstance(hsl2, apt_pkg.HashStringList)
        self.assertGreater(len(hsl2), 0)
        self.assertTrue(hsl2.verify_file(apt_pkg.__file__))


if __name__ == "__main__":
    unittest.main()



import os
import subprocess
import unittest


class PackagePyCodeStyleTestCase(unittest.TestCase):

    def test_pycodestyle(self):
        res = 0
        py_dir = os.path.join(os.path.dirname(__file__), "..")
        res += subprocess.call(
            ["pycodestyle",
             # disable for now:
             # E125 continuation line does not distinguish itself from
             #      next logical line
             # E126 continuation line over-indented for hanging indent
             # E127 continuation line over-indented for visual indent
             # E128 continuation line under-indented for visual indent
             # E129 continuation line does not distinguish itself from
             #      next logical line
             # E265 block comment should start with '# '
             # E402 module level import not at top of file (breaks tests)
             # W504 line break after binary operator (that's the
             #      correct behavior)
             "--ignore=E125,E126,E127,E128,E129,E265,E402,W504",
             "--exclude", "build,tests/old",
             "--repeat", py_dir])
        if res != 0:
            self.fail("pycodestyle failed with: %s" % res)


if __name__ == "__main__":
    unittest.main()



# cdrom.py - CDROM handling
#
#  Copyright (c) 2005-2009 Canonical
#  Copyright (c) 2009 Julian Andres Klode <jak@debian.org>
#
#  Author: Michael Vogt <michael.vogt@ubuntu.com>
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation; either version 2 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA
"""Classes related to cdrom handling."""
from __future__ import print_function

try:
    from typing import Optional
except ImportError:
    pass

import glob

import apt_pkg
from apt.progress.base import CdromProgress


class Cdrom(apt_pkg.Cdrom):
    """Support for apt-cdrom like features.

    This class has several optional parameters for initialisation, which may
    be used to influence the behaviour of the object:

    The optional parameter `progress` is a CdromProgress() subclass, which will
    ask for the correct cdrom, etc. If not specified or None, a CdromProgress()
    object will be used.

    The optional parameter `mountpoint` may be used to specify an alternative
    mountpoint.

    If the optional parameter `nomount` is True, the cdroms will not be
    mounted. This is the default behaviour.
    """

    def __init__(self, progress=None, mountpoint=None, nomount=True):
        # type: (Optional[CdromProgress], Optional[str], bool) -> None
        apt_pkg.Cdrom.__init__(self)
        if progress is None:
            self._progress = CdromProgress()
        else:
            self._progress = progress
        # see if we have a alternative mountpoint
        if mountpoint is not None:
            apt_pkg.config.set("Acquire::cdrom::mount", mountpoint)
        # do not mess with mount points by default
        if nomount:
            apt_pkg.config.set("APT::CDROM::NoMount", "true")
        else:
            apt_pkg.config.set("APT::CDROM::NoMount", "false")

    def add(self, progress=None):
        # type: (Optional[CdromProgress]) -> bool
        """Add cdrom to the sources.list."""
        return apt_pkg.Cdrom.add(self, progress or self._progress)

    def ident(self, progress=None):
        # type: (Optional[CdromProgress]) -> str
        """Identify the cdrom."""
        return apt_pkg.Cdrom.ident(self, progress or self._progress)

    @property
    def in_sources_list(self):
        # type: () -> bool
        """Check if the cdrom is already in the current sources.list."""
        cd_id = self.ident()
        if cd_id is None:
            # FIXME: throw exception instead
            return False
        # Get a list of files
        src = glob.glob(apt_pkg.config.find_dir("Dir::Etc::sourceparts") + '*')
        src.append(apt_pkg.config.find_file("Dir::Etc::sourcelist"))
        # Check each file
        for fname in src:
            with open(fname) as fobj:
                for line in fobj:
                    if not line.lstrip().startswith("#") and cd_id in line:
                        return True
        return False
